
buycoin_params = {
    "allowed_cryptocurrency": ["bitcoin", "litecoin", "naira_token", "usd_coin", "ethereum", "usd_tether"],
    "status": ["open", "completed"],
    "operation": ["buy", "sell", "create", "send", "balance"],
    "side": ["buy", "sell"]

}
